@@ -22,7 +22,6 @@ impl RuntimeConfig {
 
     // public for testing purposes in filters.rs
     pub(crate) fn create_clap_app() -> clap::App<'static> {
-        #[allow(clippy::redundant_closure)]
         App::new(env!("CARGO_PKG_NAME"))
             .version(env!("CARGO_PKG_VERSION"))
             .author("alpha-tango-kilo <git@heyatk.com>")
@@ -32,14 +31,14 @@ impl RuntimeConfig {
                 Arg::new("non-interactive")
                     .short('n')
                     .long("non-interactive")
-                    .about("Disables interactive features (always picks the first result)")
+                    .help("Disables interactive features (always picks the first result)")
                     .requires("search_term"),
             )
             .arg(
                 Arg::new("number_of_results")
                     .short('r')
                     .long("results")
-                    .about("The maximum number of results to show from IMDb")
+                    .help("The maximum number of results to show from IMDb")
                     .takes_value(true)
                     .conflicts_with("non-interactive")
                     .validator(|s| s.parse::<usize>().map_err(|_| ClapError::NotUsize)),
@@ -48,15 +47,12 @@ impl RuntimeConfig {
                 Arg::new("filter_genre")
                     .short('g')
                     .long("genre")
-                    .about("Filters results to a specific genre")
-                    .long_about(
-                        "Filters results to a specific genre\n\
+                    .help("Filters results to a specific genre (movie, series, episode)")
+                    .long_help(
+                        "Filters results to a specific genre (movie, series, episode)\n\
                     Can be given multiple arguments or passed multiple times, \
                     working as a chain of OR statements logically. \
-                    Filters are all case insensitive\n\
-                    It is STRONGLY recommended you quote genres, as most have \
-                    spaces\n\
-                    Examples: Movie, \"TV episode\", \"TV series\"",
+                    Filters are all case insensitive",
                     )
                     .takes_value(true)
                     .multiple_values(true)
@@ -66,8 +62,8 @@ impl RuntimeConfig {
                 Arg::new("filter_year")
                     .short('y')
                     .long("year")
-                    .about("Filter results to a specific year")
-                    .long_about(
+                    .help("Filter results to a specific year")
+                    .long_help(
                         "Filters results to a specific year, or range of years\n\
                     Media which has no year specified will always be included\n\
                     Ranges are fully inclusive\n\
@@ -81,29 +77,34 @@ impl RuntimeConfig {
                 Arg::new("format")
                     .short('f')
                     .long("format")
-                    .about("Change output format to desired standard")
-                    .long_about(
+                    .help("Change output format to desired standard")
+                    .long_help(
                         "Change output format to desired standard\n\
                     Formats are only available if you opted-IN at installation\n\
                     All the formats imdb-id can support are: json, yaml",
                     )
-                    // Clippy thinks this closure is redundant
                     .validator(|s| OutputFormat::try_from(s))
                     .takes_value(true),
             )
             .arg(
                 Arg::new("search_term")
-                    .about("The title of the movie/show you're looking for")
+                    .help("The title of the movie/show you're looking for")
                     .takes_value(true)
                     .multiple_values(true),
             )
             .arg(
                 Arg::new("api_key")
                     .long("api-key")
-                    .about("Your OMDb API key")
-                    .long_about("Your OMDb API key (overrides saved value if present)")
+                    .alias("apikey")
+                    .help("Your OMDb API key")
+                    .long_help("Your OMDb API key (overrides saved value if present)")
                     .takes_value(true),
             )
+            .after_long_help("ENVIRONMENT VARIABLES:\n    \
+            IMDB_ID_MAX_REQUESTS_PER_SEARCH\n            \
+            Adjusts the limit on the number \
+            of requests per search. Default is 10\
+            ")
     }
 
     fn process_matches(clap_matches: &ArgMatches) -> Result<Self> {
@@ -187,7 +188,7 @@ pub enum OutputFormat {
 impl TryFrom<&str> for OutputFormat {
     type Error = ClapError;
 
-    fn try_from(s: &str) -> std::result::Result<Self, ClapError> {
+    fn try_from(s: &str) -> Result<Self, ClapError> {
         let variant = match s.to_ascii_lowercase().as_str() {
             "human" | "plain" => Human,
             "json" => Json,
@@ -206,16 +207,16 @@ mod unit_tests {
     #[test]
     fn help() {
         let clap = RuntimeConfig::create_clap_app();
-        let m = clap
+        let err = clap
             .try_get_matches_from(vec![env!("CARGO_PKG_NAME"), "-h"])
             .unwrap_err();
-        assert_eq!(m.kind, clap::ErrorKind::DisplayHelp);
+        assert_eq!(err.kind, clap::ErrorKind::DisplayHelp);
 
         let clap = RuntimeConfig::create_clap_app();
-        let m = clap
+        let err = clap
             .try_get_matches_from(vec![env!("CARGO_PKG_NAME"), "--help"])
             .unwrap_err();
-        assert_eq!(m.kind, clap::ErrorKind::DisplayHelp);
+        assert_eq!(err.kind, clap::ErrorKind::DisplayHelp);
     }
 
     #[test]
@@ -365,7 +366,8 @@ mod unit_tests {
         let config = RuntimeConfig::process_matches(&m).unwrap();
         assert_eq!(config.format, OutputFormat::Json);
 
-        if cfg!(feature = "yaml") {
+        #[cfg(feature = "yaml")]
+        {
             let clap = RuntimeConfig::create_clap_app();
             let m = clap
                 .try_get_matches_from(vec![
@@ -396,7 +398,8 @@ mod unit_tests {
         let config = RuntimeConfig::process_matches(&m).unwrap();
         assert_eq!(config.format, OutputFormat::Json);
 
-        if cfg!(feature = "yaml") {
+        #[cfg(feature = "yaml")]
+        {
             let clap = RuntimeConfig::create_clap_app();
             let m = clap
                 .try_get_matches_from(vec![
