@@ -1,28 +1,44 @@
 use crate::omdb::SearchResult;
-use crate::RunError::InvalidYearRange;
-use crate::{Result, Year};
+use crate::{Result, RunError, YearParseError};
 use clap::ArgMatches;
+use lazy_static::lazy_static;
+use serde::de::Error;
+use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use smallvec::{smallvec, SmallVec};
+use std::cmp::{max, min};
+use std::fmt;
+use std::ops::RangeInclusive;
 use std::str::FromStr;
+use Genre::*;
+
+lazy_static! {
+    // I'm so sorry, this is my compromise for easily getting the current year
+    static ref CURRENT_YEAR: u16 = {
+        use std::time::SystemTime;
+        let timestamp = humantime::format_rfc3339(SystemTime::now())
+            .to_string();
+        timestamp.split_once('-').unwrap().0.parse().expect("Bad current year")
+    };
+}
 
 #[derive(Debug)]
 #[cfg_attr(test, derive(Eq, PartialEq))]
 pub struct Filters {
-    genres: SmallVec<[String; 3]>,
-    years: Option<Year>,
+    pub genres: SmallVec<[Genre; 3]>,
+    pub years: Option<Year>,
 }
 
 impl Filters {
     pub fn new(clap_matches: &ArgMatches) -> Result<Self> {
         let mut genres = SmallVec::new();
         if let Some(vs) = clap_matches.values_of("filter_genre") {
-            vs.for_each(|s| genres.push(s.into()));
+            for v in vs {
+                genres.push(Genre::from_str(v)?);
+            }
         }
 
         let years = match clap_matches.value_of("filter_year") {
-            Some(year_str) => {
-                Some(Year::from_str(year_str).map_err(InvalidYearRange)?)
-            }
+            Some(year_str) => Year::from_str(year_str)?.into(),
             None => None,
         };
 
@@ -31,35 +47,31 @@ impl Filters {
 
     pub fn allows(&self, search_result: &SearchResult) -> bool {
         let year_matches = match &self.years {
-            Some(year) => match (year, &search_result.year) {
-                (Year::Single(a), Year::Single(b)) => a == b,
-                (
-                    Year::Range { start: None, .. },
-                    Year::Range { start: None, .. },
-                ) => true,
-                (
-                    Year::Range { end: None, .. },
-                    Year::Range { end: None, .. },
-                ) => true,
-                (Year::Range { .. }, Year::Range { start, end }) => {
-                    start.map_or(false, |s| year.contains(s))
-                        || end.map_or(false, |s| year.contains(s))
-                }
-                (Year::Single(a), Year::Range { .. }) => {
-                    search_result.year.contains(*a)
-                }
-                (Year::Range { .. }, Year::Single(b)) => year.contains(*b),
-            },
+            Some(year) => year.contains(&search_result.year),
             None => true,
         };
 
         let genre_matches = self.genres.is_empty()
             || self.genres.iter().any(|allowed_genre| {
-                search_result.media_type.eq_ignore_ascii_case(allowed_genre)
+                allowed_genre.eq(search_result.media_type.as_str())
             });
         //println!("{:?}\n^ year matches: {}, genre matches: {}", search_result, year_matches, genre_matches);
         year_matches && genre_matches
     }
+
+    pub fn combinations(&self) -> usize {
+        // `Other` genres are ignored as they aren't used when creating request
+        // bundles
+        let genres = max(
+            self.genres
+                .iter()
+                .filter(|g| !matches!(g, Other(_)))
+                .count(),
+            1,
+        );
+        let years = self.years.as_ref().map(|year| year.0.len()).unwrap_or(1);
+        genres * years
+    }
 }
 
 impl Default for Filters {
@@ -71,10 +83,227 @@ impl Default for Filters {
     }
 }
 
+// Limitation: series' are assumed to end in the current year
+// Fortunately due to the Display impl the user won't see this
+#[derive(Debug, Clone)]
+#[cfg_attr(test, derive(Eq, PartialEq))]
+pub struct Year(pub(crate) RangeInclusive<u16>);
+
+impl Year {
+    const SEPARATORS: [char; 2] = ['-', '–'];
+
+    pub fn contains(&self, year: &Year) -> bool {
+        self.0.start() <= year.0.end() && year.0.start() <= self.0.end()
+    }
+
+    fn is_single(&self) -> bool {
+        self.0.start() == self.0.end()
+    }
+}
+
+impl FromStr for Year {
+    type Err = YearParseError;
+
+    // WARNING: not all separators are one byte, this must not be assumed!
+    fn from_str(year_str: &str) -> Result<Self, Self::Err> {
+        use YearParseError::*;
+
+        // TODO: warnings if year has to be modified
+        match year_str.split_once(&Year::SEPARATORS[..]) {
+            Some((start_str, end_str)) => {
+                let mut start = if !start_str.is_empty() {
+                    // Make sure arg isn't bigger than current year
+                    min(u16::from_str(start_str)?, *CURRENT_YEAR)
+                } else {
+                    0
+                };
+                let mut end = if !end_str.is_empty() {
+                    // Make sure arg isn't bigger than current year
+                    min(u16::from_str(end_str)?, *CURRENT_YEAR)
+                } else if start_str.is_empty() {
+                    return Err(NoYearsSpecified);
+                } else {
+                    *CURRENT_YEAR
+                };
+                // Save the user from their silliness
+                if end < start {
+                    use std::mem;
+                    mem::swap(&mut start, &mut end);
+                }
+                Ok(Year(start..=end))
+            }
+            None => {
+                // Should be just a year we can parse
+                let year = min(u16::from_str(year_str)?, *CURRENT_YEAR);
+                Ok(Year(year..=year))
+            }
+        }
+    }
+}
+
+impl Serialize for Year {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_str(&self.to_string())
+    }
+}
+
+impl<'de> Deserialize<'de> for Year {
+    fn deserialize<D>(d: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let s = String::deserialize(d)?;
+        Year::from_str(&s).map_err(|e| {
+            D::Error::custom(format!("could not parse field as year ({:?})", e))
+        })
+    }
+}
+
+impl fmt::Display for Year {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        if self.is_single() {
+            write!(f, "{}", self.0.start())
+        } else {
+            let start = *self.0.start();
+            let end = *self.0.end();
+            if start != 0 {
+                write!(f, "{start}")?;
+            }
+            write!(f, "-")?;
+            if end != *CURRENT_YEAR {
+                write!(f, "{end}")?;
+            }
+            Ok(())
+        }
+    }
+}
+
+// These are the OMDb API supported genres to filter by
+// Serialize and Deserialize and implemented by hand
+#[derive(Debug, Clone)]
+#[cfg_attr(test, derive(Eq, PartialEq))]
+pub enum Genre {
+    Movie,
+    Series,
+    Episode,
+    Other(String),
+}
+
+impl Genre {
+    fn as_str(&self) -> &str {
+        self.as_ref()
+    }
+}
+
+impl AsRef<str> for Genre {
+    fn as_ref(&self) -> &str {
+        match self {
+            Movie => "movie",
+            Series => "series",
+            Episode => "episode",
+            Other(s) => s,
+        }
+    }
+}
+
+impl FromStr for Genre {
+    type Err = RunError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s.to_ascii_lowercase().as_str() {
+            "movie" => Ok(Movie),
+            "series" => Ok(Series),
+            "episode" => Ok(Episode),
+            _ => Err(RunError::InvalidGenre(s.to_owned())),
+        }
+    }
+}
+
+impl From<&str> for Genre {
+    fn from(s: &str) -> Self {
+        Self::from_str(s).unwrap_or_else(|_| Other(s.to_owned()))
+    }
+}
+
+impl PartialEq<str> for Genre {
+    fn eq(&self, other: &str) -> bool {
+        other.eq_ignore_ascii_case(self.as_str())
+    }
+}
+
+impl fmt::Display for Genre {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{}", self.as_ref())
+    }
+}
+
+// Serialize with Genre.as_str
+impl Serialize for Genre {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_str(self.as_str())
+    }
+}
+
+// Deserialize with From<str> for Genre
+impl<'de> Deserialize<'de> for Genre {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        String::deserialize(deserializer).map(|s| Genre::from(s.as_str()))
+    }
+}
+
 #[cfg(test)]
-mod unit_tests {
+mod filters_unit_tests {
+    use crate::{Filters, Genre::*, Year};
+    use smallvec::smallvec;
+
+    #[test]
+    fn combinations() {
+        let filters = vec![
+            Filters {
+                genres: smallvec![],
+                years: None,
+            },
+            Filters {
+                genres: smallvec![Movie, Series],
+                years: None,
+            },
+            Filters {
+                genres: smallvec![],
+                years: Some(Year(1960..=1970)),
+            },
+            Filters {
+                genres: smallvec![Movie, Episode],
+                years: Some(Year(1980..=2000)),
+            },
+        ];
+        let expected: Vec<usize> = vec![1, 2, 11, 42];
+
+        filters
+            .iter()
+            .zip(expected)
+            .for_each(|(filters, expected)| {
+                assert_eq!(
+                    filters.combinations(),
+                    expected,
+                    "Expected {expected} combination(s) from {:#?}",
+                    filters,
+                );
+            });
+    }
+
     mod creation {
-        use crate::{Filters, RuntimeConfig, Year, Year::*};
+        use crate::filters::CURRENT_YEAR;
+        use crate::Genre::*;
+        use crate::{Filters, RuntimeConfig, Year};
         use smallvec::smallvec;
 
         #[test]
@@ -84,14 +313,14 @@ mod unit_tests {
                 .try_get_matches_from(vec![
                     env!("CARGO_PKG_NAME"),
                     "-g",
-                    "TV Episode",
+                    "episode",
                 ])
                 .unwrap();
             let filters = Filters::new(&clap_matches).unwrap();
             assert_eq!(
                 filters,
                 Filters {
-                    genres: smallvec!["TV Episode".into()],
+                    genres: smallvec![Episode],
                     years: None,
                 }
             );
@@ -101,7 +330,7 @@ mod unit_tests {
                 .try_get_matches_from(vec![
                     env!("CARGO_PKG_NAME"),
                     "-g",
-                    "TV Episode",
+                    "Episode",
                     "Movie",
                 ])
                 .unwrap();
@@ -109,7 +338,7 @@ mod unit_tests {
             assert_eq!(
                 filters,
                 Filters {
-                    genres: smallvec!["TV Episode".into(), "Movie".into()],
+                    genres: smallvec![Episode, Movie],
                     years: None,
                 }
             );
@@ -130,7 +359,7 @@ mod unit_tests {
                 filters,
                 Filters {
                     genres: smallvec![],
-                    years: Some(Single(1980)),
+                    years: Some(Year(1980..=1980)),
                 }
             );
 
@@ -147,10 +376,7 @@ mod unit_tests {
                 filters,
                 Filters {
                     genres: smallvec![],
-                    years: Some(Year::Range {
-                        start: Some(1980),
-                        end: Some(2010),
-                    }),
+                    years: Some(Year(1980..=2010)),
                 }
             );
 
@@ -167,10 +393,7 @@ mod unit_tests {
                 filters,
                 Filters {
                     genres: smallvec![],
-                    years: Some(Year::Range {
-                        start: Some(1980),
-                        end: None,
-                    }),
+                    years: Some(Year(1980..=*CURRENT_YEAR)),
                 }
             );
 
@@ -187,10 +410,7 @@ mod unit_tests {
                 filters,
                 Filters {
                     genres: smallvec![],
-                    years: Some(Year::Range {
-                        start: None,
-                        end: Some(2010),
-                    }),
+                    years: Some(Year(0..=2010)),
                 }
             );
         }
@@ -210,10 +430,7 @@ mod unit_tests {
                 filters,
                 Filters {
                     genres: smallvec![],
-                    years: Some(Year::Range {
-                        start: Some(1980),
-                        end: Some(2010),
-                    }),
+                    years: Some(Year(1980..=2010)),
                 }
             );
         }
@@ -228,18 +445,15 @@ mod unit_tests {
                     "1980-2010",
                     "-g",
                     "Movie",
-                    "Video",
+                    "episode",
                 ])
                 .unwrap();
             let filters = Filters::new(&clap_matches).unwrap();
             assert_eq!(
                 filters,
                 Filters {
-                    genres: smallvec!["Movie".into(), "Video".into()],
-                    years: Some(Year::Range {
-                        start: Some(1980),
-                        end: Some(2010),
-                    }),
+                    genres: smallvec![Movie, Episode],
+                    years: Some(Year(1980..=2010)),
                 }
             );
         }
@@ -247,6 +461,7 @@ mod unit_tests {
 
     mod filtering {
         use crate::omdb::SearchResult;
+        use crate::Genre::*;
         use crate::{Filters, Year};
         use once_cell::sync::Lazy;
         use smallvec::smallvec;
@@ -259,81 +474,75 @@ mod unit_tests {
                     SearchResult {
                         title: "Kingsman: The Secret Service".into(),
                         imdb_id: "tt2802144".into(),
-                        media_type: "Movie".into(),
-                        year: Year::Single(2014),
+                        media_type: Movie,
+                        year: Year(2014..=2014),
                     },
                     SearchResult {
                         title: "The King's Man".into(),
                         imdb_id: "tt6856242".into(),
-                        media_type: "Movie".into(),
-                        year: Year::Single(2021),
+                        media_type: Movie,
+                        year: Year(2021..=2021),
                     },
                     SearchResult {
                         title: "Kingsman: The Golden Circle".into(),
                         imdb_id: "tt4649466".into(),
-                        media_type: "Movie".into(),
-                        year: Year::Single(2017),
+                        media_type: Movie,
+                        year: Year(2017..=2017),
                     },
                     SearchResult {
                         title: "Kingsman: The Secret Service Revealed".into(),
                         imdb_id: "tt5026378".into(),
                         media_type: "Video".into(),
-                        year: Year::Single(2015),
+                        year: Year(2015..=2015),
                     },
                     SearchResult {
                         title: "Kingsman: Inside the Golden Circle".into(),
                         imdb_id: "tt7959890".into(),
                         media_type: "Video".into(),
-                        year: Year::Single(2017),
+                        year: Year(2017..=2017),
                     },
                     SearchResult {
                         title: "Kingsman: Bespoke Lessons for Gentlemen Spies"
                             .into(),
                         imdb_id: "tt6597836".into(),
                         media_type: "TV Series".into(),
-                        year: Year::Single(2015),
+                        year: Year(2015..=2015),
                     },
                     SearchResult {
                         title: "King's Man".into(),
                         imdb_id: "tt1582211".into(),
-                        media_type: "Movie".into(),
-                        year: Year::Single(2010),
+                        media_type: Movie,
+                        year: Year(2010..=2010),
                     },
                     SearchResult {
                         title: "All the King's Men".into(),
                         imdb_id: "tt0405676".into(),
-                        media_type: "Movie".into(),
-                        year: Year::Single(2006),
+                        media_type: Movie,
+                        year: Year(2006..=2006),
                     },
                     SearchResult {
                         title: "The Kingsman".into(),
                         imdb_id: "tt13332408".into(),
                         media_type: "TV Episode".into(),
-                        year: Year::Single(2020),
+                        year: Year(2020..=2020),
                     },
                     SearchResult {
                         title: "All the King's Men".into(),
                         imdb_id: "tt0041113".into(),
-                        media_type: "Movie".into(),
-                        year: Year::Single(1949),
+                        media_type: Movie,
+                        year: Year(1949..=1949),
                     },
                     SearchResult {
                         title: "Black Mirror".into(),
                         imdb_id: "tt2085059".into(),
-                        media_type: "Series".into(),
-                        year: Year::Range {
-                            start: Some(2016),
-                            end: None,
-                        },
+                        media_type: Series,
+                        year: Year(2016..=2021),
                     },
                     SearchResult {
                         title: "Seinfeld".into(),
                         imdb_id: "tt0098904".into(),
-                        media_type: "Series".into(),
-                        year: Year::Range {
-                            start: Some(1989),
-                            end: Some(1998),
-                        },
+                        media_type: Series,
+                        year: Year(1989..=1998),
                     },
                 ]
             });
@@ -364,7 +573,7 @@ mod unit_tests {
         #[test]
         fn genre_single() {
             let test = Filters {
-                genres: smallvec!["Movie".into()],
+                genres: smallvec![Movie],
                 years: None,
             };
             let results = [
@@ -387,7 +596,7 @@ mod unit_tests {
         #[test]
         fn genre_multiple() {
             let test = Filters {
-                genres: smallvec!["Movie".into(), "Video".into()],
+                genres: smallvec![Movie, "Video".into()],
                 years: None,
             };
             let results = [
@@ -434,10 +643,7 @@ mod unit_tests {
         fn years() {
             let test = Filters {
                 genres: smallvec![],
-                years: Some(Year::Range {
-                    start: Some(2020),
-                    end: None,
-                }),
+                years: Some(Year(2020..=2021)),
             };
             let results = [
                 false, true, false, false, false, false, false, false, true,
@@ -447,10 +653,7 @@ mod unit_tests {
 
             let test = Filters {
                 genres: smallvec![],
-                years: Some(Year::Range {
-                    start: Some(1950),
-                    end: Some(2010),
-                }),
+                years: Some(Year(1950..=2010)),
             };
             let results = [
                 false, false, false, false, false, false, true, true, false,
@@ -462,11 +665,8 @@ mod unit_tests {
         #[test]
         fn mixed() {
             let test = Filters {
-                genres: smallvec!["Movie".into()],
-                years: Some(Year::Range {
-                    start: Some(1950),
-                    end: Some(2010),
-                }),
+                genres: smallvec![Movie],
+                years: Some(Year(1950..=2010)),
             };
             let results = [
                 false, false, false, false, false, false, true, true, false,
@@ -475,11 +675,8 @@ mod unit_tests {
             assert_eq!(&get_outcomes(&test), &results);
 
             let test = Filters {
-                genres: smallvec!["Movie".into(), "TV Episode".into()],
-                years: Some(Year::Range {
-                    start: Some(2010),
-                    end: None,
-                }),
+                genres: smallvec![Movie, "TV Episode".into()],
+                years: Some(Year(2010..=2021)),
             };
             let results = [
                 true, true, true, false, false, false, true, false, true,
@@ -489,3 +686,48 @@ mod unit_tests {
         }
     }
 }
+
+#[cfg(test)]
+mod year_unit_tests {
+    use super::Year;
+    use super::CURRENT_YEAR;
+    use lazy_static::lazy_static;
+    use std::ops::RangeInclusive;
+    use std::str::FromStr;
+
+    const STR_INPUTS: [&str; 7] = [
+        "1999",
+        "-1999",
+        "1999–",
+        "1920-1925",
+        "1000-800",
+        "2020–2021",
+        "2048-",
+    ];
+
+    lazy_static! {
+        static ref YEARS: [RangeInclusive<u16>; 7] = [
+            1999..=1999,
+            0..=1999,
+            1999..=*CURRENT_YEAR,
+            1920..=1925,
+            800..=1000,
+            2020..=2021,
+            *CURRENT_YEAR..=*CURRENT_YEAR,
+        ];
+    }
+
+    #[test]
+    fn from_str() {
+        STR_INPUTS
+            .iter()
+            .map(|s| Year::from_str(s).expect("Year should have parsed"))
+            .zip(YEARS.iter())
+            .for_each(|(a, b)| assert_eq!(a.0, *b));
+    }
+
+    #[test]
+    fn from_str_invalid() {
+        Year::from_str("-").unwrap_err();
+    }
+}
