@@ -1,7 +1,6 @@
-use crate::get_api_key;
 use crate::omdb::test_api_key;
 use crate::Result;
-use reqwest::blocking::Client;
+use crate::{get_api_key, ApiKeyError};
 use serde::{Deserialize, Serialize};
 use std::fs;
 use std::fs::OpenOptions;
@@ -14,8 +13,8 @@ pub struct OnDiskConfig {
 }
 
 impl OnDiskConfig {
-    pub fn new_from_prompt(client: &Client) -> Result<Self> {
-        let api_key = get_api_key(client)?;
+    pub fn new_from_prompt() -> Result<Self> {
+        let api_key = get_api_key()?;
         Ok(OnDiskConfig { api_key })
     }
 
@@ -26,8 +25,8 @@ impl OnDiskConfig {
             .truncate(true)
             .open(OnDiskConfig::config_path())?;
 
-        let bytes = serde_json::to_vec(&self)?;
-        file.write_all(&bytes)
+        let ser = serde_json::to_string_pretty(&self)?;
+        file.write_all(ser.as_bytes())
     }
 
     pub fn load() -> std::io::Result<Self> {
@@ -36,14 +35,14 @@ impl OnDiskConfig {
         Ok(config)
     }
 
-    pub fn check(&self, client: &Client) -> std::result::Result<(), String> {
-        test_api_key(&self.api_key, client)
+    pub fn check(&self) -> Result<(), ApiKeyError> {
+        test_api_key(&self.api_key)
     }
 
-    pub fn validate(&mut self, client: &Client) -> Result<()> {
-        if let Err(why) = self.check(client) {
-            eprintln!("{}", why);
-            self.api_key = get_api_key(client)?;
+    pub fn validate(&mut self) -> Result<()> {
+        if let Err(why) = self.check() {
+            eprintln!("{why}");
+            self.api_key = get_api_key()?;
         }
         Ok(())
     }
